@@ -47,14 +47,19 @@
 
 (tm-define (style-includes? x y)
   (:require (and (== x "beamer")
-                 (in? y (list "title-bar" "bluish" "varsession"))))
+                 (in? y (list "title-bar" "bluish" "framed-session"))))
   #t)
 
 (tm-define (style-includes? x y)
   (:require (and (== x "old-beamer")
-                 (in? y (list "framed-title" "ridged-paper" "varsession"))))
+                 (in? y (list "framed-title" "ridged-paper" "framed-session"))))
   #t)
 
+(tm-define (screens-buffer?)
+  (with t (buffer-tree)
+    (and (tree-is? t 'document)
+         (tree-is? t :last 'screens))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Dynamic movements for fold tags and switches
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -242,7 +247,7 @@
 ;; Specific types of switches
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define (alternative-context? t)
+(tm-define (alternative-context? t)
   (alternative-tag? (tree-label t)))
 
 (tm-define (switch-select t i)
@@ -343,9 +348,6 @@
 (tm-define (overlay-context? t)
   (overlay-tag? (tree-label t)))
 
-(define (tree->number t)
-  (if (tree-atomic? t) (string->number (tree->string t)) 0))
-
 (tm-define (make-overlays l)
   (if (== l 'overlays)
       (insert-go-to `(,l "1" "1" (document "")) '(2 0 0))
@@ -599,7 +601,7 @@
                ((== mode :last)
                 (dynamic-extremal t #t))))))
 
-(define (dynamic-operate t mode)
+(tm-define (dynamic-operate t mode)
   (when (tree-compound? t)
     (cond ((tree-is? t 'traversed)
            (when (!= mode :var-last)
@@ -626,19 +628,19 @@
 (define (dynamic-first-alternative-list l)
   (if (null? l) #f
       (with r (dynamic-first-alternative (car l))
-	(or r (dynamic-first-alternative-list (cdr l))))))
+        (or r (dynamic-first-alternative-list (cdr l))))))
 
 (define (dynamic-first-alternative t)
   (cond ((and (alternative-context? t) (> (tm-arity t) 1)) t)
-	((overlays-context? t)
+        ((overlays-context? t)
          (tree-set! t 0 "1")
          (tree-assign-node! t 'overlays-range)
          t)
         ((and (tree-is? t 'overlays-range)
               (< (tree->number (tree-ref t 0))
                  (tree->number (tree-ref t 1)))) t)
-	((not (tm-compound? t)) #f)
-	(else (dynamic-first-alternative-list (tm-children t)))))
+        ((not (tm-compound? t)) #f)
+        (else (dynamic-first-alternative-list (tm-children t)))))
 
 (define (dynamic-alternative-keep-first slide)
   (and-with t (dynamic-first-alternative slide)
@@ -661,29 +663,73 @@
     (tree-assign-node! t 'slide))
   (if (and (tm-func? t 'slide 1) (dynamic-first-alternative t))
       (let* ((p (tree-up t))
-	     (i (tree-index t)))
-	(tree-insert p (+ i 1) (list (tree-copy t)))
-	(let* ((v  (tree-ref p i))
-	       (w  (tree-ref p (+ i 1))))
-	  (dynamic-alternative-keep-first v)
-	  (dynamic-alternative-keep-other w)
-	  (dynamic-make-slide v)
-	  (dynamic-make-slide w)))
+             (i (tree-index t)))
+        (tree-insert p (+ i 1) (list (tree-copy t)))
+        (let* ((v  (tree-ref p i))
+               (w  (tree-ref p (+ i 1))))
+          (dynamic-alternative-keep-first v)
+          (dynamic-alternative-keep-other w)
+          (dynamic-make-slide v)
+          (dynamic-make-slide w)))
       (dynamic-operate (tree-ref t 0) :var-expand)))
 
+(define (keep-shown! t)
+  (if (alternative-context? t)
+      (map ;; Prune hidden children
+       (lambda (i) (if (tm-func? (tree-ref t i) 'hidden)
+                       (tree-remove! t i 1)))
+       (reverse (.. 0 (tm-arity t)))))
+  (if (and (tm-compound? t) (> (tm-arity t) 0))
+      (for-each keep-shown! (tree-children t))))
+
+(define (create-slide scr)
+  (if (not (tree? scr)) (tree 'slide '(document "")) ;; just in case
+      (with t (tree-copy scr)
+        (keep-shown! t)
+        (tree-assign-node! t 'slide)
+        t)))
+
+(define (process-screen scr)
+  (cons (create-slide scr)
+        (begin
+          (dynamic-traverse-buffer :next)
+          (if (tm-func? scr 'hidden) '()
+              (process-screen scr)))))
+
+(define (list->tree label l)
+  (tree-insert (tree label) 0 l))
+
+(define (screens->slides t)
+  (if (not (tm-func? t 'screens)) (tree 'document "")
+      (with f (lambda (scr) (list->tree 'document (process-screen scr)))
+        ;; (system-wait "Generating slides" "please wait") ;crashes if printing
+        ;; Insert fake screen at the end
+        (tree-insert! t (tree-arity t) 
+                      (list (tree 'hidden '(document ""))))
+        (dynamic-operate-on-buffer :first)
+        ;; Notice that we don't process the last (fake) screen
+        (list->tree 'screens (map f (cDr (tree-children t)))))))
+
 (tm-define (dynamic-make-slides)
-  (add-style-package "slides")
   (init-default "page-medium" "page-type" "page-width" "page-height"
                 "page-width-margin" "page-height-margin"
                 "page-odd" "page-even" "page-right"
                 "par-width" "page-odd-shift" "page-even-shift"
                 "page-top" "page-bot" "page-height-margin")
-  (let* ((t (buffer-tree))
-         (c (tree-children t)))
-    (when (and (tm-func? t 'document) (switch-context? (cAr c)))
-      (tree-assign-node (cAr c) 'document)
-      (tree-set! t `(document ,@(cDr c) ,@(tree-children (cAr c))))
-      (for-each dynamic-make-slide (tree-children t)))))
+  (add-style-package "slides")
+  (if (preference-on? "texmacs->pdf:expand slides")
+      (let* ((t (buffer-tree))
+             (c (tree-children t)))
+        (when (and (tm-func? t 'document) (switch-context? (cAr c)))
+          (tree-assign-node (cAr c) 'document)
+          (tree-set! t `(document ,@(cDr c) ,@(tree-children (cAr c))))
+          ;; (system-wait "Generating slides" "please wait") ;crashes if printing
+          (for-each dynamic-make-slide (tree-children t))))
+      (with l (select (buffer-tree) '(screens))
+        (and (nnull? l)
+             (let* ((scrns (car l))
+                    (slides (screens->slides scrns)))
+               (tree-set! scrns slides))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Global filtering of switches
@@ -801,7 +847,7 @@
        (or (dynamic-traverse (car l) mode)
            (dynamic-traverse-list (cdr l) mode))))
 
-(define (dynamic-traverse t mode)
+(tm-define (dynamic-traverse t mode)
   (cond ((tree-atomic? t) #f)
         ((tree-is? t 'traversed)
          (dynamic-traverse-traversed t mode))
@@ -874,25 +920,3 @@
       (begin
         (switch-insert-at t :var-next)
         (make-item))))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Balloons
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-
-(tm-define (show-texmacs-balloon body balloon)
-  (with w (widget-texmacs-output balloon '(style "generic"))
-    (with (x1 y1 x2 y2) (tree-bounding-rectangle body)
-      (show-balloon w x1 (- y1 (* 5 256))))))
-
-(tm-define (show-mouse-over-balloon)
-  (:secure #t)
-  (with-action body
-    (and-with balloon (tree-ref body :next)
-      (show-texmacs-balloon body balloon))))
-
-(tm-define (show-help-balloon)
-  (:secure #t)
-  (with-action body
-    (and-with balloon (tree-ref body :next)
-      (show-texmacs-balloon
-       body `(colored-frame "pastel yellow" ,balloon)))))
